# PROGRAM VERSION & CASE PARAMETERS

programversion = 0.11

dynastyname = "Romanov"
startyear   =  1613
endyear     =  1917 # 1917


      ###############################################################################
      #                                                                             #
      # SIMPLE DYNASTY BY EDEG SIUMLATOR                                            #
      #_____________________________________________________________________________#
      #                                                                             #
      # Copyright 2013 - 1016 by Mark Ciotola                                       #
      # Distributed under the terms of the GNU General Public license               #
      # Last revised on 19 December 2015                                            #
      # Contact information: mpaciotola@gmail.com                                   #
      # Further information at: http://www.fastentropy.org                          #
      #_____________________________________________________________________________#
      #                                                                             #
      # Developed with Ruby 2.2.1p85                                                #
      #                                                                             #
      ###############################################################################


# SET-UP

  # Require

    require "csv"

  # Include libraries
    include Math

  prompt = "> "


# CREATE BUBBLE CLASS

  class Bubble

  attr_accessor :secondlawmult, :period, :efficiency, :consumption, :consumptionrate, :production, :productionrate, :initpotential

	# Initialize parameters

    def initialize(attributes = {})
      
      @period = 1
      @initpotential = 500 #1000000.0
      @secondlawmult = 0.0207
      @efficiency = 1.0  # A nominal setting; will be calculated later
      @consumption = 0.0
      @production = 1.0  # Must equal 1 if initial production rate is equal to 1
      @productionrate = 0.0 # A nominal setting; will be calculated later
      @consumptionrate = 1.0
      
    end

	# Define actions

    def secondlawmult
      return @secondlawmult
    end

    def period
      return @period
    end

    def efficiency
      return @efficiency
    end

    def consumption
      return @consumption
    end

    def production
      return @production
    end

    def productionrate
      return @productionrate
    end
    
    def consumptionrate
        return @consumptionrate
    end

  end


# CREATE BUBBLE CLASS INSTANCE

  bubble = Bubble.new


# SIMULATION BANNER

  puts "\n\n"
  puts "###########################################################################"
  puts "#                                                                         #"
  puts "# SIMPLE DYNASTY BY EDEG SIUMLATOR version " + programversion.to_s + "                           #"
  puts "#_________________________________________________________________________#"
  puts "#                                                                         #"
  puts "# Copyright 2011-2015 by Mark Ciotola                                     #"
  puts "# Dist under GNU General Public License                                   #"
  puts "#                                                                         #"
  puts "###########################################################################"
  puts ""


# INTRODUCTION AND INSTRICTIONS

  puts "\n\n"
  puts "The program generates a historical dynasty bubble by discounting exponential"
  puts "growth by efficiency, utilizing a generic potential."
  puts "\n\n"
  puts "Active parameters are potential, second law multiplier, and initial consumption rate."
  puts "Display parameters are start and end year, as long as end year is set sufficiently late."
  puts "Parameters are entered directly into the code."
  puts "\n\n"

# User Inputs Simulation Parameters

  puts "\n\n"

  periods = endyear - startyear


# CREATE OUTPUT FILE

  dynasty_output_file = "dyn-edeg-v" + programversion.to_s + "_" + dynastyname + "_" + startyear.to_s + "-" + endyear.to_s + "_" + bubble.secondlawmult.to_s

  dynasty_output_file = dynasty_output_file + ".csv"


# Display Initial Parameter Values

  puts "\n\n"
  puts "PARAMETERS AND INITIAL VALUES: \n\n"
  puts sprintf "Periods: \t\t %3d" , periods
  puts sprintf "Multiplier Factor: \t %4.4f" , bubble.secondlawmult.to_f


# Display Simulation Banner
 
  puts "\n\n"  
  puts "RESULTS: \n\n"

  puts "YEAR  \t\tCONS RAT\tCONSUMPT\tEFFICIENCY\t\tPRODRATE\t\tGROWTH %\t\tPRODUCTN"
  puts "------\t\t--------\t--------\t----------\t\t--------\t\t--------\t\t-------- \n"


# GENERATE AND DISPLAY SIMULATION RESULTS (this is a loop)

  # Set up initial loop variables

    year = bubble.period + startyear
    growthpercent = 0.0
    potential = bubble.initpotential

  bubble.productionrate = bubble.consumptionrate * bubble.efficiency * bubble.secondlawmult


  while (bubble.period < (periods.to_f + 1)) and ( bubble.productionrate > 0) do
    
    consumptionrateold = bubble.consumptionrate

    bubble.efficiency =  (potential/bubble.initpotential) * bubble.secondlawmult
    
    efficiencydisplay =  bubble.efficiency * 100.00

    bubble.consumptionrate = bubble.production
    
    bubble.productionrate = bubble.consumptionrate * bubble.efficiency
    
    bubble.consumption = bubble.consumption + bubble.consumptionrate
        
    bubble.production = bubble.production + bubble.productionrate

    growthpercent = ((bubble.consumptionrate/consumptionrateold) - 1) * 100.0
    
    relativepower = bubble.productionrate / bubble.secondlawmult


    # Output
      
      # Set up output string
    
        per = bubble.period
        dcn = bubble.consumptionrate
        raw = bubble.consumption
        eff = efficiencydisplay
        dpr = relativepower  # bubble.productionrate
        net = bubble.production
        grw = growthpercent
        yr = year
    
        mystring = ("%d \t%13.3f \t  %13.3f \t\  %6.4f \t %13.4f \t\t %13.3f \t\t %13.3f")
        
      # Send output to screen
        
        puts sprintf mystring, yr.to_s, dcn.to_s, raw.to_s, eff.to_s, dpr.to_s, grw.to_s, net.to_s
    
      # Append to output file

        CSV.open(dynasty_output_file, "a+") do |csv|
          csv << [yr.to_s, raw.to_s, eff.to_s, dpr.to_s]
        end


    # Update Variables

      bubble.period = bubble.period + 1
      year = startyear + bubble.period
      potential = potential - bubble.consumptionrate
    
    
  end


# INDICATE LOOP HAS EXECUTED AND THAT SIMULATION IS COMPLETED.

  puts "\nDone. \n"
  puts "\nSimulation is completed. \n\n"



###############################################################################
#                                                                             #
# CHANGE LOG                                                                  #
#                                                                             #
###############################################################################

#_____________________________________________________________________________
#
# Version 0.09 - Started 14 November 2015.
#_____________________________________________________________________________

# Cleaned out unused code for user entry choices, etc.

#_____________________________________________________________________________
#
# Version 0.10 / 0.11 - Started 12 December 2015.
#_____________________________________________________________________________

# Fixed potential leakage and year calculations.

# Also includes relative power.


###############################################################################
#                                                                             #
# NOTES                                                                       #
#                                                                             #
###############################################################################

#  Need 2nd Law Efficiency curve
#  2nd Law Efficiency increases due to economies of scale
#  Costs of maintaining economies of scale


#  Total energy production
#  (Population x diet calories) + additional sources

#  Need to input data file (start date, end date, population information, diet)

# Possible Future Improvements: choice of amount of time constants for decay.

# Please feel free to contact the author with comments or further suggestions for improvements.


######### MORE NOTES v0.08+

# Graph production versus consumption to see what it looks like.

# bubble.efficiency =  ( 1.0 - (bubble.consumption/bubble.initpotential)) * secondlawmult



